# 核心概念

在正式进行开发 Rax 项目前，你最好能了解以下核心概念。如果你熟悉 React，并对以下所有概念了如指掌，你可以选择跳过阅读以下部分。

* **组件(Components)** - 组件是可复用的 UI 元素， Rax 内置提供了如 **`Text`** 和 **`Image`** 这样的跨容器组件。Rax 使用组件来封装界面模块，整个界面就是一个大组件，开发过程就是不断优化和拆分界面组件、构造整个组件树的过程。

* **属性(Props)** -  **`Props`** 就是组件的属性。展示一个组件时只需要指定 **`Props `**作为 JSX 节点的属性。组件很少需要对外公开方法，唯一的交互途径就是 **`Props`**。这使得使用组件就像使用函数一样简单，给定一个输入，组件给定一个界面输出。

* **状态(State)** - 组件内部维持的状态数据称为 **`state`** ，它是组件的当前状态。可以把组件简单看成一个”状态机”，根据 **`state`** 呈现不同的 UI 展示。一旦 **`state`** 被更改，组件就会自动调用自身的 `render` 函数重新渲染 UI，这个更改 **`state`** 的动作会通过 `this.setState `方法来触发：

```js
this.setState({ value: 10 })
```

* **事件(Events)** - Rax 中绑定事件的方式和在 HTML 中绑定事件类似，使用**驼峰式命名**指定要绑定的事件属性为组件定义的一个方法：
```jsx
<TextInput onInput={ (event) => this.setState({ text: event.value }) } />
```

* **Flexbox 布局** - Rax 使用 `flexbox` 规则来描述组件。 `Flexbox` 意为"弹性布局"，用来为盒状模型提供最大的灵活性。你可以查阅 [样式处理](./guide/style) 来了解 `flexbox` 的基本布局方法。
* **样式** - 我们可以使用对象的方式来描述 CSS 中的样式，并传递给组件的 **`style`** 属性，唯一的区别是带连词号(-)的属性需要用驼峰写法代替。例如：
```jsx
 <View style={{ width: 100, height: 100, backgroundColor: 'skyblue' }} />
```
虽然支持，但 Rax 不推荐使用内联对象的方式来应用样式，你可以查阅 [样式处理](./guide/style) 来查看在 Rax 中使用样式的最佳实践。
